// FIBER PCI INTERFACE
//  For PCI v3.0 -- WITH 23-STAGE SCRAM/DSCRAM
//  Author              Greg Burley
//  Organization        OCIW
//  Revision Level      3.6c
//  Target Device       EPM7256AETC144-5

// Module Declaration
module pci4w (DQ, D, CLK, RESET, ERXD, ERXFS, ERXCK, ETXD, ETXFS, ETXCK,
	PTWR, PTADR, PTNUM, DXFR, EF, LF, OPT3V, ADDINT, TXBUSY, RXBUSY, 
	WEN, WCLK, REN, RCLK, LD, OE, ERST, FRST, FTXD, FTXCK, FRXD, FRXCK, 
	REFCLK, ADCLK, TX_LD, SDATA, RDATA, LED2, LED3, G0, G1, G2, G3, G4, 
	G5, G6, G7, G8, G9, G10, G11, G12, G13, G14, G15);

input	[15:0] DQ;					// PCI data bus
output	[15:0] D;					// FIFO data bus
input	CLK, RESET;					// on-board clock and reset
input	ERXD, ERXFS, ERXCK;			// incoming serial data
output	ETXD, ETXFS, ETXCK;			// outgoing serial data
input	PTWR, PTADR; 				// Add-on bus write + address
input	[1:0] PTNUM;				// Add-on bus region select
input	DXFR;						// Add-on data transfer strobe
input	EF;							// FIFO empty flag
input   LF; 						// Optical loss of signal -- not used
input   OPT3V;						// optical=1 wire=0
output	ADDINT; 					// Add-on Interrupt
output  TXBUSY, RXBUSY;				// serial shiftreg busy flags
output	WEN, WCLK, REN, RCLK;		// fifo write and read signals
output	LD, OE;						// fifo enables
output	ERST, FRST;					// reset signals
output	FTXD, FTXCK;				// optical output data and clock
input	FRXD, FRXCK;				// optical input data and clock
output	REFCLK;						// CDR chip reference clock 
output  ADCLK;						// Add-on bus clock
output	G0,G1,G2,G3,G4,G5,G6,G7,G8,G9,G10,G11,G12,G13,G14,G15;
output	LED2, LED3;					// LED drive signals

output	TX_LD;						// TX shift register load signal
output	SDATA, RDATA;				// unscrambled shiftreg output and input

wire [22:0] SCRAM;
wire [22:0] DSCRAM;
wire [17:0] TSHIFT, TSHIFT_d;
wire [15:0] RSHIFT;
wire [15:0] DQMUX;
wire [5:0] RST, RST_eq;
wire [4:0] TCNT, TCNT_eq;
wire [4:0] RCNT, RCNT_eq;
wire [2:0] CNT;
wire RDATA, RDATA_in;
wire SCRAM_in;

// 	CLOCK CIRCUIT + TXCLK AND ADCLK OUTPUT
lpm_counter	cnt(.clock(CLK), .q(CNT));
	defparam cnt.lpm_width=3;
	defparam cnt.lpm_direction="UP";

assign FTXCK = CLK;
assign ADCLK = CNT[1];		// CLK / 4 
assign REFCLK = CNT[2];		// CLK / 8
assign RCLK = ~ADCLK;	
assign WCLK = ~FRXCK;

// 	NON-FUNCTIONING SIGNALS
assign ETXCK = 1'bz;
assign ETXD = 1'bz;
assign ETXFS = 1'bz;

//	SCRAMBLER + DESCRAMBLER DESCRIPTIONS
//	SCRAM.shiftin = TXD $ SCRAM.q[4] $ SCRAM.q[22]; (23-STAGE)
lpm_shiftreg scram(.clock(CLK), .q(SCRAM), .shiftin(SCRAM_in));
	defparam scram.lpm_width=23;
	defparam scram.lpm_direction="LEFT";

assign SCRAM_in = SDATA ^ SCRAM[4] ^ SCRAM[22];
assign FTXD = SCRAM[22];

lpm_shiftreg dscram(.clock(FRXCK), .q(DSCRAM), .shiftin(FRXD));
	defparam dscram.lpm_width=23;
	defparam dscram.lpm_direction="LEFT";

assign RDATA_in = FRXD ^ DSCRAM[4] ^ DSCRAM[22];
dff	u1 (.clk(FRXCK), .d(RDATA_in), .q(RDATA));


// 	TX SHIFT REGISTER
lpm_shiftreg tshift(.clock(CLK), .q(TSHIFT), .data(TSHIFT_d),
	.load(TX_LD), .shiftin(1'b0), .shiftout(SDATA));
	defparam tshift.lpm_width=18;
	defparam tshift.lpm_direction="RIGHT";

assign DQMUX = (RESET) ? DQ[15:0] : 16'b0;
assign TSHIFT_d[17:0] = {~RESET, DQMUX[15:0], 1'b1};
assign TX_LD = ((PTWR & ~DXFR & TCNT_eq[0]) | RST_eq[1]);

//	SEND DUMMY WORDS WHEN RESET ASSERTED
lpm_counter	rst(.clock(CLK), .q(RST), .eq(RST_eq), 
	.cnt_en(~RESET), .sclr(RESET));
	defparam rst.lpm_width=6;
	defparam rst.lpm_direction="DOWN";

// 	TX FRAME_SYNC/BUSY GENERATOR CIRCUIT
lpm_counter	tcnt(.clock(CLK), .q(TCNT), .sset(TX_LD), .eq(TCNT_eq), 
	.cnt_en(~TCNT_eq[0]));
	defparam tcnt.lpm_width=5;
	defparam tcnt.lpm_direction="DOWN";
	defparam tcnt.lpm_svalue=18;

assign TXBUSY = ~TCNT_eq[0] | TX_LD; 

// 	RX SHIFT REGISTERS
lpm_shiftreg rshift (.clock(FRXCK), .q(RSHIFT), .shiftin(RDATA));
	defparam rshift.lpm_width=16;
	defparam rshift.lpm_direction="RIGHT";

assign D[15:0] = RSHIFT[15:0];

// 	GENERATE SIGNAL TO LOAD FIFO FROM RX SHIFT REG (WCLK = RXCLK)
lpm_counter	rcnt(.clock(FRXCK), .q(RCNT), .sset(RDATA & RCNT_eq[0]),
 	.eq(RCNT_eq), .cnt_en(~RCNT_eq[0]));
	defparam rcnt.lpm_width=5;
	defparam rcnt.lpm_direction="DOWN";
	defparam rcnt.lpm_svalue=17;

assign RXBUSY = ~RCNT_eq[0]; 

dff	u2(.clk (FRXCK), .d(~RCNT_eq[2]), .q(WEN));

//	GENERATE FIFO READ_ENABLE (RCLK = !CLK)
assign REN = ~( ~PTWR & ~DXFR & (PTNUM[1:0]=='b00));
assign OE = REN;
assign ADDINT = ~EF;

//	LED SIGNALS -- BOTH YELLOW
assign LED2 = TXBUSY;
assign LED3 = RXBUSY;

//	OTHER/DUMMY SIGNALS
assign ERST = RESET;
assign FRST = RESET;
assign LD = 1'b1;

assign G0 = 1'bz;
assign G1 = 1'bz;
assign G2 = 1'bz;
assign G3 = 1'bz;
assign G4 = SDATA;
assign G5 = RDATA;
assign G6 = 1'bz;
assign G7 = 1'bz;
assign G8 = 1'bz;
assign G9 = 1'bz;
assign G10 = 1'bz;
assign G11 = 1'bz;
assign G12 = 1'bz;
assign G13 = 1'bz;
assign G14 = 1'bz;
assign G15 = 1'bz;

endmodule

